class PgDmDesignSkillBootstrapOptions extends PgDmDesignSkill {
    constructor(version) {
        super('bootstrap', 'Bootstrap options');

        this.version = version;

        this.addControl('sizer',
            {
                name: 'Text size',
                type: 'range',
                range_min: -5,
                range_max: 5,
                magic_range_min: -2,
                magic_range_max: 2,
                dm_locked: true,
                default_value: 0,
                helptext: `<p>Set the size of text and margin / padding spacers.</p>`
            }
        )

        this.addControl('gutter',
            {
                name: 'Grid spacing',
                type: 'range',
                range_min: -5,
                range_max: 5,
                range_marks: true,
                dm_locked: true,
                default_value: 0,
                helptext: `<p>Set the amount of space between columns and column padding.</p>`
            }
        )

        this.addControl('roundness',
            {
                name: 'Roundness',
                type: 'range',
                range_min: 0,
                range_max: 4,
                range_marks: true,
                helptext: `<p>Set the roundness of cards, input fields, pills...</p>`
            }
        )

        this.addControl('buttonRoundness',
            {
                name: 'Button roundness',
                type: 'range',
                range_min: 0,
                range_max: 4,
                range_marks: true,
                helptext: `<p>Set the button roundness.</p>`
            }
        )

        this.addControl('shadows',
            {
                name: 'Shadows',
                type: 'range',
                range_min: 0,
                range_max: 4,
                range_marks: true,
                helptext: `<p>Set shadows for cards, navbars...</p>`
            }
        )

        this.addControl('contrast',
            {
                name: 'Contrast',
                type: 'range',
                range_min: 0,
                range_max: 2,
                range_marks: true,
                //default_value: 1,
                dm_no_lock: 'placeholder',
                helptext: `<p>Minimum contrast for calculating light/dark color combinations, for example the color of button labels. Values range is 3, 4.5 and 7 (acceptable values for WCAG 2.0).</p>`
            }
        )

        this.addControl('gradients',
            {
                name: 'Gradients',
                type: 'checkbox',
                value: '1',
                helptext: `<p>Enable gradients on buttons, alerts...</p>`
            }
        )

        if(this.version === '4') {
            this.addControl('responsive_fonts',
                {
                    name: 'Responsive font sizes',
                    type: 'checkbox',
                    value: '1',
                    helptext: `<p>Enable responsive font sizes to scale font sizes across viewport sizes.</p>`
                }
            )
        }
    }

    populateDefaultParams() {
        this.setParam('contrast', '0', true);
    }

    onSurpriseMe(done) {
        if(!this.isLocked('gradients')) {
            this.setParam('gradients', Math.random() < 0.3 ? '1' : '');
        }
        /*
        if(!this.isLocked('shadows')) {
            this.setParam('shadows', Math.random() < 0.3 ? '1' : '');
        }

         */
        if(!this.isLocked('responsive_fonts')) {
            this.setParam('responsive_fonts', Math.random() < 0.5 ? '1' : '');
        }
        this.surpriseMeForRangeControls(done);
    }

    onGetCSS(part) {
        var _this = this;
        var code = '';

        var round = function(from, unit, param, def) {
            var f = _this.getParam(param || 'roundness');
            if(f === null) f = def || 1;
            return (Math.round((f > 1 ? (from + (from / 0.333) * (f - 1)) : from * f) * 100) / 100) + unit;
        }

        switch(part) {

            case 'init':
                var sizer = this.getParam('sizer');
                if(sizer !== 1) {
                    code += 'html { font-size: ' + (100 + (5 * sizer)) + '% !important; }';
                }
                code += "\n" + this.setVar('grid-gutter-width', (30 + (this.getParam('gutter') * 6) + 'px'));

                code += this.setVar('border-radius', round(0.25, 'rem', null, 2))
                code += this.setVar('border-radius-lg', round(0.3, 'rem', null, 2))
                code += this.setVar('border-radius-sm', round(0.2, 'rem', null, 2))

                code += this.setVar('dropdown-border-radius', '.25rem !default')

                code += this.setVar('btn-border-radius', round(0.25, 'rem', 'buttonRoundness'))
                code += this.setVar('btn-border-radius-lg', round(0.3, 'rem', 'buttonRoundness'))
                code += this.setVar('btn-border-radius-sm', round(0.2, 'rem', 'buttonRoundness'))

                if(this.getParam('buttonRoundness') >= 2 ) {
                    code += this.setVar('btn-padding-x', '1rem');
                    code += this.setVar('btn-padding-x-sm', '0.75rem');
                    code += this.setVar('btn-padding-x-lg', '1.25rem');
                }

                code += this.setVar('enable-shadows', this.getParam('shadows') ? 'true' : 'false')

                if(this.version === '4') {
                    code += this.setVar('enable-responsive-font-sizes', this.getParam('responsive_fonts') ? 'true' : 'false')
                } else if(this.version === '5') {
                    code += this.setVar('enable-negative-margins', 'true');
                }
                code += this.setVar('enable-gradients', this.getParam('gradients') ? 'true' : 'false')

                const contrasts = [3, 4.5, 7];
                code += this.setVar('min-contrast-ratio', contrasts[parseInt(this.getParam('contrast') || 1)]);

                if(this.getParam('colorful')) {
                    code += this.setVar('headings-color', '$color1');
                }

                break;

            case 'rules':

                if(this.getParam('shadows')) {
                    var sf = this.getParam('shadows');
                    var y = 0.25 + 0.25 / 4 * (sf-1);
                    var b = 0.75 + 0.75 / 4 * (sf-1);
                    var a = 0.05 + 0.2 / 4 * (sf-1);
                    code += '.navbar, .card { box-shadow: 0 ' + y + 'rem ' + b + 'rem rgba(0, 0, 0, ' + a + '); }\r\n';
                }
                break;
        }
        return code;
    }
}

class PgDmDesignSkillColorsForBootstrap extends PgDmDesignSkillColors {
    constructor(version, colors) {
        super('bs' + version + '.colors', colors, function() {
            var css = '';
            colors.forEachColor(function(key, val) {
                css += `$${key}: ${val};
`;
            })

            return css;
        });
        this.version = version;
    }

    populateDefaultParams(keep_existing) {

        if(!keep_existing) this.colors.clear();

        if(this.version === '4') {
            this.colors.add('primary', '#007bff');
            this.colors.add('secondary', '#6c757d');
            this.colors.add('info', '#17a2b8', false);
            this.colors.add('success', '#28a745', true);
            this.colors.add('warning', '#ffc107', true);
            this.colors.add('danger', '#dc3545', true);
            this.colors.add('white', 'white', true);
            this.colors.add('light', '#f8f9fa', true);
            this.colors.add('dark', '#343a40', true);
        } else {
            this.colors.add('primary', '#0d6efd');
            this.colors.add('secondary', '#6c757d');
            this.colors.add('info', '#0dcaf0', false);
            this.colors.add('success', '#198754', true);
            this.colors.add('warning', '#ffc107', true);
            this.colors.add('danger', '#dc3545', true);
            this.colors.add('white', 'white', true);
            this.colors.add('light', '#f8f9fa', true);
            this.colors.add('dark', '#212529', true);
        }

        super.populateDefaultParams();
    }
}

class PgDmDesignProviderForBootstrap extends PgDmDesignProvider {

    constructor(f, version) {
        super('bs' + version, 'Bootstrap ' + version + ' Design');
        this.framework = f;
        this.prefix = 'bs' + version;

        this.version = version;
        var _this = this;

        this.on_engine_ready_funcs = []

        this.colors = new PgDmDesignColors();
        //this.colors.can_edit = true;

        //this.defineColors();

        this.addSkills();

        pinegrow.addEventHandler('on_bootstrap_theme_setup_done', function() {
            //restart the engine in order to switch from generate to inject mode
            _this.destroyEngine();

            if(pgDmStore.isDesignEnabled()) {
                pgDmStore.update();
            }
        })
    }

    addSkills() {
        var _this = this;

        var colors_skill = new PgDmDesignSkillColorsForBootstrap(this.version, this.colors);

        this.addSkill( colors_skill );

        this.fonts_skill_main = new PgDmDesignSkillFonts(false, colors_skill);
        this.addSkill( this.fonts_skill_main );
        this.fonts_skill_headings = new PgDmDesignSkillFonts(true, colors_skill);
        this.addSkill( this.fonts_skill_headings );
        this.addSkill( new PgDmDesignSkillBackground(colors_skill, 'BS') );
        this.addSkill( new PgDmDesignSkillBootstrapOptions( this.version ) );
    }

    getActiveFonts() {
        var r = []
        this.fonts_skill_main.getActiveFonts(r);
        this.fonts_skill_headings.getActiveFonts(r);
        return r;
    }

    getEngine(done) {
        var _this = this;
        if(!this.engine) {
            this.engine = new PgDmEngineForBootstrap(function(engine) {
                _this.on_engine_ready_funcs.forEach(function(func) {
                    func(engine);
                })
                _this.on_engine_ready_funcs = [];
                done && done(engine);
            }, this.framework);
        } else if(this.engine.isReady()) {
            done(this.engine);
        } else {
            _this.on_engine_ready_funcs.push(done);
        }
    }

    destroyEngine() {
        this.engine && this.engine.destroy();
        this.engine = null;
    }
}

class PgDmDesignProviderForBootstrap4 extends PgDmDesignProviderForBootstrap {

    constructor(f) {
        super(f, '4');
    }

}

class PgDmDesignProviderForBootstrap5 extends PgDmDesignProviderForBootstrap {

    constructor(f) {
        super(f, '5');
    }
}


class PgDmEngineForBootstrap extends PgDmEngine {

    constructor(on_engine_ready, f) {
        super(on_engine_ready);

        var _this = this;

        var use_fast = false;
        var is_ready = false;

        this.css_mode = 'generate';//'inject';
        //this.css_mode = 'inject';

        var path = require('path');

        var p = pinegrow.getCurrentProject();
        var out_url;
        this.custom_cs_url = null;
        this.out_url = null;


        if(p) {
            //detect the mode
            var custom_file = isApp() ? p.getAbsolutePath('bootstrap_theme/custom.scss') : null;
            if(isApp() && crsaIsFileExist(custom_file)) {
                this.css_mode = 'inject';
                this.custom_cs_url = crsaMakeUrlFromFile(custom_file);
            } else {
                out_url = p.getAbsoluteUrl('bootstrap_theme/bootstrap.css');
            }
        } else {
            var page = pinegrow.getSelectedPage();
            out_url = require('url').resolve(page.url, 'bootstrap_theme/bootstrap.css');
        }
        this.out_url = out_url;
        this.customCs = null;

        this.busy = false;
        this.update_after_busy = false;

        this.with_outcs_func_list = [];

        var done = function() {
            _this.is_ready = true;

            _this.with_outcs_func_list.forEach(function(func) {
                func();
            })
            _this.with_outcs_func_list = [];

            on_engine_ready && on_engine_ready(_this);

            _this.update();
        }

        if(this.css_mode === 'generate') {

            var outcs = this.outcs = PgCSSStylesheet();
            outcs.fast = use_fast;
            outcs.loaded = true;
            outcs.ignoreInSave = true;

            pinegrow.stylesheets.addCrsaStylesheet(outcs);

            outcs.setUrl(out_url);

            if(crsaIsFileUrl(outcs.url) || !isApp()) {
                outcs.localFile = crsaMakeFileFromUrl(outcs.url);
                if(!isApp() && p && !p.hasContentForUrl(out_url)) {
                    //prepare an empty file otherwise it fails
                    p.setContentOfUrl(out_url, '');
                }
                outcs.load(outcs.url, function() {}, false, true, true);
            }

            var dummy_outcs = this.dummy_outcs = PgCSSStylesheet();
            dummy_outcs.ignoreInSave = true;

            var cs = this.cs = PgSCSSStylesheet();
            cs.includeSourceMap = false;
            cs.dontAddIds = true;
            cs.ignoreInSave = true;

            var source_scss = 'bootstrap_theme/custom.scss';
            //source_scss = 'bootstrap_theme/test.scss';
            cs.load(f.getResourceUrl(source_scss), function () {
                _this.customCs = cs;
                if (done) {
                    done.call(_this);
                }
            }, null, null, dummy_outcs );

            if(!isApp()) {
                cs.onMapImportRelativeUrl = function(rel_url) {
                    if(rel_url === 'bootstrap/_bootstrap.scss') {
                        rel_url = 'bootstrap/bootstrap.scss';
                    }
                    return rel_url;
                }
            }

            cs.outFile = outcs.localFile;
        } else {
            setTimeout(function() {
                done && done();
            }, 10)
        }
    }


    onGetCSS(part) {
        var css = '';

        if(part == 'init' || part == 'import') {
            var o = {
                css: '',
                cs_url: this.out_url
            }
            pinegrow.dispatchEvent('on_dm_get_css', null, part, o);
            css += o.css;
        }

        if(part == 'rules') {
            var o = {
                css: '',
                cs_url: this.out_url
            }
            pinegrow.dispatchEvent('on_dm_get_css', null, part, o);
            css += o.css;
        }

        return css;
    }

    whenOutCSIsAvailable(func) {
        if(!this.outcs) {
            this.with_outcs_func_list.push(func);
        } else {
            func();
        }
    }

    activateOnPage(page, done) {
        var _this = this;

        if(this.css_mode == 'generate') {
            this.whenOutCSIsAvailable(function() {
                var replacePgel = null;
                if(page.sourceNode) {
                    if (!page.stylesheets.has(_this.outcs)) {
                        page.sourceNode.withoutEvents(function () {
                            page.sourceNode.find('link').forEach(function (pgel) {
                                if (pgel.getAttribute('rel') == 'stylesheet') {
                                    var href = pgel.getAttribute('href');
                                    if (href && href.match(/bootstrap(\.min|)/i)) {
                                        replacePgel = pgel;
                                    }
                                }
                            })

                            page.stylesheets.attachTo(_this.outcs, null, function () {
                                pinegrow.changeManager.didChange(null, 'Add generated stylesheet', null, page);
                                $('body').trigger('crsa-stylesheets-changed');
                                page.refresh();
                                done && done();
                            }, replacePgel);
                        })
                        page.setPageChanged(true);
                    } else {
                        done && done();
                    }
                } else {
                    done && done();
                }
            })
        } else {
            done && done();
        }
    }


    update(done) {

        var _this = this;

        if (!pinegrow.getSelectedPage()) return;
        if (!this.customCs && this.css_mode === 'generate') return;

        if (this.busy) {
            _this.update_after_busy = true;
            return;
        }

        var code = this.onGetCSS('import');
        code += this.onGetCSS('init');

        if (this.css_mode === 'generate') {
            code += `
@import "bootstrap/bootstrap";
`;
        }

        var extra_code = this.onGetCSS('rules') || '';

        if (this.css_mode !== 'inject') {
            code += extra_code;
        }

        var compiledCs = null;

        if(this.css_mode === 'inject') {
            _this.customCs = pinegrow.stylesheets.getByUrl(this.custom_cs_url, false);
            if(_this.customCs) {
                _this.update_after_busy = false;
                _this.busy = true;

                var source = _this.customCs.getSource();
                source = pgUpdateGeneratedSectionContent(source, 'Theme Init', code, 'prepend');
                source = pgUpdateGeneratedSectionContent(source, 'Additional Rules', extra_code, 'append');

                _this.customCs.includeSourceMap = false;

                compiledCs = _this.customCs.compiledStylesheets[0];

                compiledCs.ignore_imports_in_apply = true;
                _this.customCs.setSource(source, function () {
                    //var cs = customCs;
                    //cs.compiledStylesheet.resetError();
                    pinegrow.dispatchEvent('on_css_source_changed', null, {
                        list: [_this.customCs],
                        ignore_in_undo: true
                    });

                    pinegrow.dispatchEvent('on_css_stylesheet_changed', null, compiledCs);

                    _this.busy = false;
                    if (_this.update_after_busy) {
                        _this.update(done);
                        return;
                    }
                    pinegrow.refreshPageChangedStatusForAllPages(true);

                    done && done();

                });//, false, true, true /* fast delayed */)

                var fonts = pgDmStore.design.fonts.getFontUrls();
                _this.addFontsToPageViews(compiledCs, fonts);
            } else {
                pinegrow.showQuickError(`${_this.custom_cs_url} is not loaded.`);
            }

        } else {
            _this.update_after_busy = false;
            _this.busy = true;

            //compiledCs = cs.compiledStylesheets[0];
            compiledCs = this.outcs;

            _this.customCs.setSource(code, function () {
                _this.cs.regenerateCssFromCompilableSource(function (compilationError, renderResult) {
                    if (compilationError) {
                        console.log(compilationError);
                        _this.busy = false;
                        if(_this.update_after_busy) {
                            _this.update(done);
                            return;
                        }
                        done && done();
                    } else {
                        compiledCs.resetError();

                        _this.outcs.ignore_imports_in_apply = true;
                        _this.outcs.setSource(renderResult.css, function() {
                            _this.busy = false;
                            if(_this.update_after_busy) {
                                _this.update(done);
                                return;
                            }
                            pinegrow.refreshPageChangedStatusForAllPages(true);

                            pinegrow.dispatchEvent('on_css_stylesheet_changed', null, compiledCs);

                            done && done();

                        }, null, null, true /* fast delayed */)

                    }
                })
            }, false, true)

            var fonts = pgDmStore.design.fonts.getFontUrls();
            _this.addFontsToPageViews(compiledCs, fonts);
        }
    }

    saveCSS() {
        var outfile = crsaMakeFileFromUrl(this.out_url);
        if(this.css_mode === 'generate' && outfile) {
            this.outcs.save(outfile);
            this.outcs.setChanged(false);
        }
    }

    destroy() {
        if(this.css_mode === 'generate') {
            this.outcs.remove();
            pinegrow.stylesheets.removeStylesheetByKey(this.outcs.url);

            this.cs.remove();
            pinegrow.stylesheets.removeStylesheetByKey(this.cs.url);

            this.dummy_outcs.remove();
            pinegrow.stylesheets.removeStylesheetByKey(this.dummy_outcs.url);
        }
    }
}