class PgComposerManager {
    constructor() {
        this.instances_map = {}
        this.unused_map = {}
        this.types = {}
    }

    registerType(type, cls) {
        this.types[type] = cls;
    }

    unused(c) {
        if(!this.unused_map[c.type]) {
            this.unused_map[c.type] = [];
        }
        this.unused_map[c.type].push(c);
    }

    used(c) {
        if(this.unused_map[c.type]) {
            const idx = this.unused_map[c.type].indexOf(c);
            (idx >= 0) && this.unused_map[c.type].splice(idx, 1);
        }
    }

    registerInstance(c) {
        if(!this.instances_map[c.type]) {
            this.instances_map[c.type] = [];
        }
        this.instances_map[c.type].push(c);
    }

    getInstance(type) {
        if(this.unused_map[type] && this.unused_map[type].length) {
            return this.unused_map[type].pop();
        } else {
            return new this.types[type]();
        }
    }
}

const pgComposerManager = new PgComposerManager();

class PgComposerComponent {
    constructor(type) {
        this.type = type;
        this.id = null;
        this.data = null;
        this.element = null;
        this.map = {};
        this.cache = {};
        this.children_map = {};

        this.manager = pgComposerManager;

        this.manager.registerInstance(this);
    }

    setData(data) {
        this.data = data;
    }

    detach() {
        this.element.parentNode.removeChild(this.element);
        this.manager.unused(this);
        for(let name in this.children_map) {
            this.children_map[name].forEach(function(c) {
                c.detach();
            })
            this.children_map[name] = [];
        }
    }

    create(name, tag, attrs, children) {
        const e = document.createElement(tag);
        if(attrs)
        {
            for(let attr in attrs)
            {
                e.setAttribute(attr, attrs[attr]);
            }
        }
        if(children) {
            children.forEach(function(ch) {
                e.appendChild(ch);
            })
        }
        if(name) this.map[name] = e;
        return e;
    }

    text(text) {
        return document.createTextNode(text);
    }

    setHtml(name, html) {
        const key = name + '__html';
        if(html !== this.cache[key]) {
            this.map[name].innerHTML = html;
            this.cache[key] = html;
        }
    }

    addClass(name, cls) {
        const el = this.map[name];
        !el.classList.contains(cls) && el.classList.add(cls);
    }

    removeClass(name, cls) {
        const el = this.map[name];
        el.classList.contains(cls) && el.classList.remove(cls);
    }

    setAttribute(name, attr, value) {
        const el = this.map[name];
        if(el.getAttribute(attr) !== value) {
            el.setAttribute(attr, value);
        }
    }

    removeAttribute(name, attr) {
        const el = this.map[name];
        if(el.hasAttribute(attr)) {
            el.removeAttribute(attr);
        }
    }

    append(name, child_el) {
        const el = this.map[name];
        if(child_el.parentNode !== el) {
            el.appendChild(child_el);
        }
    }

    setChildren(name, list) {
        const _this = this;
        this.children_map[name] && this.children_map[name].forEach(function(c) {
            c.detach();
        })

        list.forEach(function(c) {
            _this.map[name].appendChild(c.element);
        })
        this.children_map[name] = list;
    }

    getElement(name) {
        return this.map[name];
    }

    objToString(obj) {
        if(Array.isArray(obj)) {
            var r = '[';
            for(var i = 0; i < obj.length; i++) {
                r += `${this.objToString(obj[i])},\n`;
            }
            r += ']';
            return r;
        } else if(typeof obj === 'object') {
            var r = '{';
            for(var key in obj) {
                var lab = key;
                r += `${lab}: ${this.objToString(obj[key])},\n`;
            }
            r += '}';
            return r;
        } else if(typeof obj === 'number' || typeof obj === 'string') {
            return obj;
        }
    }

    didChange(name, val) {
        const key = name + '__value';
        const old_val = this.cache[key] || null;
        if(old_val !== null) {
            const val_string = this.objToString(val);
            if(old_val !== val_string) {
                this.cache[key] = val_string;
                return true;
            } else {
                return false;
            }
        } else {
            if(val === null) {
                return false;
            } else {
                return true;
            }
        }
    }
}

(function() {

    const cssValueParser = new PgGetCSSValueHTML();

    class PgComposerCSSProp extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSProp');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-prop'
            },[
                this.create('label', 'div', {
                    class: 'pgc-css-prop-label'
                }),
                this.create('value', 'div', {
                    class: 'pgc-css-prop-value'
                }),
                this.create('remove', 'i', {
                    class: 'icon icon-close'
                })
            ])
        }

        setData(data) {
            super.setData(data);
            this.setHtml('label', data.prop);
            this.setHtml('value', cssValueParser.getHTML(data.value));
        }
    }

    class PgComposerCSSComment extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSComment');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-comment'
            })
        }

        setData(data) {
            super.setData(data);
            this.setHtml('top', data.text);
        }
    }

    class PgComposerCSSAtRule extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSAtRule');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-atrule'
            }, [
                this.create('name', 'span', {
                    class: 'pgc-css-atrule-name'
                }),
                this.create('params', 'span', {
                    class: 'pgc-css-atrule-params'
                })
            ])
        }

        setData(data) {
            super.setData(data);
            this.setHtml('name', data.rule.name);
            this.setHtml('params', data.rule.params);
            const ml = 10 * data.level;
            this.setAttribute('top', 'style', 'margin-left:' + ml + 'px');
        }
    }

    class PgComposerCSSAddProp extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSAddProp');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-add-prop'
            }, [
                this.text('Add')
            ])
        }

        setData(data) {
            super.setData(data);
        }
    }

    class PgComposerCSSRule extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSRule');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-rule'
            },[
                this.create('atrule', 'div', {
                    class: 'pgc-css-prop-atrule'
                }),
                this.create('selector', 'div', {
                    class: 'pgc-css-prop-selector'
                }),
                this.create('props', 'div', {
                    class: 'pgc-css-rule-props'
                }),
           /*     this.create('remove', 'i', {
                    class: 'icon icon-close'
                }) */
            ])
        }

        setData(data) {
            const _this = this;

            super.setData(data);
            const node = data.node;

            if(this.didChange('rule', node)) {
                this.setHtml('selector', node.selector);

                const props = [];
                node.nodes.forEach(function(prop) {
                    let c;

                    switch(prop.type) {
                        case 'comment':
                            c =_this.manager.getInstance('PgComposerCSSComment');
                            break;
                        default:
                            c =_this.manager.getInstance('PgComposerCSSProp');
                    }
                    c.setData(prop);
                    props.push(c);
                })

                props.push(_this.manager.getInstance('PgComposerCSSAddProp'));
                this.setChildren('props', props);
            }

            const parents = []
            let parent = node.parent;
            while(parent && parent.type === 'atrule') {
                parents.unshift(parent);
                parent = parent.parent;
            }

            const atrules = []
            for(let i = 0; i < parents.length; i++) {
                const ac = _this.manager.getInstance('PgComposerCSSAtRule');
                ac.setData({rule: parents[i], level: i});
                atrules.push(ac);
            }

            this.setChildren('atrule', atrules);
            const ml = 10 * atrules.length;
            this.setAttribute('selector', 'style', 'margin-left:' + ml + 'px');
            this.setAttribute('props', 'style', 'margin-left:' + ml + 'px');

            if(pinegrow.selectedCSSNodes.isSelected(node)) {
                this.addClass('top', 'selected');
            } else {
                this.removeClass('top', 'selected');
            }
        }
    }


    class PgComposerCSSElementRules extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSElementRules');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-element-rules'
            },[
                this.create('element', 'div', {
                    class: 'pgc-css-element-rules-element'
                }),
                this.create('rules', 'div', {
                    class: 'pgc-css-element-rules-rules'
                }),
            ])
        }

        setData(data) {
            const _this = this;

            super.setData(data);
            this.setHtml('element', data.pgel.getName());

            const rules = [];
            data.rules.forEach(function(rule) {
                const c = _this.manager.getInstance('PgComposerCSSRule');
                c.setData(rule);
                rules.push(c);
            })

            this.setChildren('rules', rules);
        }
    }


    class PgComposerCSSPanel extends PgComposerComponent {
        constructor() {
            super('PgComposerCSSPanel');
            this.element = this.create('top', 'div', {
                class: 'pgc-css-panel'
            },[
                this.create('rules', 'div', {
                    class: 'pgc-css-rules'
                }),
            ])

            const $element = $(this.element);

            const scrubValue = new PgCSSEditableValue($element, 'num');

            scrubValue.onChange = function($num, val, is_done) {
                //onChangeValue($num, undefined, is_done);
            }

            const colorValue = new PgCSSEditableColor($element, 'colorbox');
            colorValue.onChange = function($color) {
                //onChangeValue($color);
            }

            const editValue = new PgCSSEditableText($element, '.pgc-css-prop-label,.pgc-css-prop-value');
            editValue.onGetValue = function($el) {
                return $el.text();
            }
        }

        setData(data) {
            const _this = this;

            super.setData(data);

            const rules = [];
            data.forEach(function(rule) {
                const c = _this.manager.getInstance('PgComposerCSSElementRules');
                c.setData(rule);
                rules.push(c);
            })

            this.setChildren('rules', rules);
        }
    }

    window.PgComposerCSSPanel = PgComposerCSSPanel;

    pgComposerManager.registerType('PgComposerCSSProp', PgComposerCSSProp);
    pgComposerManager.registerType('PgComposerCSSComment', PgComposerCSSComment);
    pgComposerManager.registerType('PgComposerCSSAtRule', PgComposerCSSAtRule);
    pgComposerManager.registerType('PgComposerCSSAddProp', PgComposerCSSAddProp);
    pgComposerManager.registerType('PgComposerCSSRule', PgComposerCSSRule);
    pgComposerManager.registerType('PgComposerCSSElementRules', PgComposerCSSElementRules);
    pgComposerManager.registerType('PgComposerCSSPanel', PgComposerCSSPanel);

    class PgComposerTester {
        constructor(element) {
            this.element = element;

            this.data = {
                rules: [
                    {
                        selector: 'h1',
                        props: [
                            {name: 'color', value: '#224466'},
                            {name: 'font-size', value: '20px'},
                            {name: 'background', value: 'url(nice.jpg)'},
                        ]
                    },
                    {
                        selector: 'h1.huge',
                        props: [
                            {name: 'font-size', value: '40px'},
                        ]
                    }
                ]
            }

            var panel = pgComposerManager.getInstance('PgComposerCSSPanel');
            this.element.appendChild(panel.element);
            panel.setData(this.data);
        }
    }


})();