var PgProjectDB = function(project) {

    var _this = this;

    var initialized = false;

    var file_name = 'projectdb.pgml';

    var assign_ids = true;

    var getFilePath = function() {
        return project.getAbsolutePath(file_name);
    }

    var getRelativeUrl = function(url) {
        return project.getRelativeUrl(url);
    }

    this.is_loaded = false;
    this.is_changed = false;

    var project_node = null;
    var docs_node;
    var root_node;

    var docs_url_map = new Map();

    this.isInitialized = function() {
        return project_node !== null;
    }

    this.init = function(skip_create) {
        if(!this.load()) {
            if(!skip_create) {
                this.create();
            } else {
                return false;
            }
        } else {
            this.is_loaded = true;
            initialized = true;
        }
        return true;
    }

    var cache_saved_pages = {}

    var undo_page = null;

    var setupUndoPage = function() {
        var page = new PgEmulatePageForUndo();
        page.sourceNode = root_node;
        root_node.emulatedPage = page;
        page.name = 'Project';

        undo_page = page;
    }

    this.load = function(project) {
        try {
            if(!project || !project.loaded_from_backend) {
                var fs = require('fs');
                var html = fs.readFileSync(getFilePath(), {encoding: 'utf8'});
            } else {
                if(project) {
                    html = project.getContentOfUrl(file_name);
                }
                if(!html) {
                    throw 'Project DB does not exist.';
                }
            }
            var p = new pgParser();
            p.assignIds = assign_ids;
            p.parse(html);

            docs_url_map.clear();

            root_node = p.rootNode;
            project_node = p.rootNode.findOne('project');
            docs_node = project_node.findOne('documents');

            docs_node.find('>document').forEach(function(doc) {
                var url = doc.getAttribute('url');
                docs_url_map.set(url, doc)

                cache_saved_pages[url] = doc.html();
            })

            setupUndoPage();

            return true;
        } catch(err) {

        }
        return false;
    }

    this.restoreSavedDocument = function(url) {
        url = getRelativeUrl(url);
        if(cache_saved_pages[url]) {
            var doc = docs_url_map.get(url);
            if(doc) {
                doc.html(cache_saved_pages[url], null, null, true /* no ids */)
            }
            this.changed();
        }
    }

    this.rememberSavedDocument = function(url) {
        url = getRelativeUrl(url);
        var doc = docs_url_map.get(url);
        if(doc) {
            cache_saved_pages[url] = doc.html()
        }
    }

    this.create = function() {
        var p = new pgParser();
        p.assignIds = assign_ids;
        p.parse('<project></project>');

        root_node = p.rootNode;
        project_node = root_node.findOne('project');
        docs_node = c('documents');
        project_node.append(docs_node);

        setupUndoPage();
    }

    this.getProjectNode = function() {
        return project_node;
    }

    this.findOne = function(sel) {
        return project_node ? project_node.findOne(sel) : null;
    }

    this.getOrCreateNode = function(tag, parent) {
        parent = parent || project_node;
        var n = parent.findOne('> ' + tag);
        if(!n) {
            n = c(tag);
            parent.append(n);
        }
        return n;
    }

    this.getDocumentNode = function(url) {
        url = getRelativeUrl(url);
        var node = docs_url_map.get(url);
        if(!node) {
            node = c('document', {
                url: url
            })
            docs_node.append(node);
            docs_url_map.set(url, node);

            node.append(c('elements'))
        }
        return node;
    }

    this.getElementNodeForPgel = function(pgel, page) {
        page = page || pgel.getPage();
        var sel = pgel.getAttr('data-pg-sel');
        return this.getElementNodeForSelector(sel, page)
    }

    this.getElementNodeForSelector = function(sel, page) {
        if(!sel) return null;

        var element = null;

        if(!element) {
            var doc = this.getDocumentNode(page.url);
            var list = doc.findOne('elements').children;
            for(var i = 0; i < list.length; i++) {
                if(list[i].getAttr('selector') === sel) {
                    element = list[i];
                    break;
                }
            }
        }
        return element;
    }

    this.getElementsWithIssues = function() {
        return project_node.find('element').filter(function(node) {
            var status = node.getAttr('status');
            if(status !== 'ok' && status !== 'unused') {
                return true;
            } else {
                return false;
            }
        })
    }

    this.createElementNode = function(attrs) {
        var node = c('element', attrs)
        return node;
    }

    this.createActionNode = function(action_id, params) {
        var node = c('action', params);
        node.attrList.unshift({
            name: 'id',
            value: action_id,
            quote: '"'
        })
        return node;
    }

    this.save = function() {
        if(!project.loaded_from_backend) {
            crsaWriteFileWithBackup(null, getFilePath(), project_node.toStringOriginal(true), 'utf8');
            this.is_changed = false;
        } else {
            pinegrow.backend.saveDelayed();
        }
    }

    this.getHTMLSource = function() {
        return project_node.toStringOriginal(true);
    }

    this.saveAsBackup = function() {
        var fs = require('fs');
        crsaCopyFileSync(fs, getFilePath(), getFilePath() + '.backup');
        fs.unlinkSync(getFilePath());
    }

    var batchUpdates = null;

    this.startBatchUpdates = function() {
        batchUpdates && this.endBatchUpdates('Project DB update');
        batchUpdates = new PgCollection();
        in_make_changes_level = 0;
    }

    this.endBatchUpdates = function(name, source) {
        if(batchUpdates && batchUpdates.getLength()) {
            pinegrow.makeChanges(batchUpdates, name, function() {});
            this.changed(null, {
                source: source
            })
        }
        batchUpdates = null;
        in_make_changes_level = 0;
    }

    var in_make_changes_level = 0;

    this.makeChanges = function(node, name, func, source) {
        node = node || root_node;

        if(batchUpdates) {
            batchUpdates.add(node);
            func();
            return;
        }

        in_make_changes_level++;

        if(in_make_changes_level === 1) {
            pinegrow.makeChanges(root_node.getPage(), name, func);
        } else {
            func();
        }

        in_make_changes_level--;

        if(in_make_changes_level === 0) {
            this.changed(null, {
                source: source
            })
        }
    }

    this.changed = function(col, info) {
        this.is_changed = true;
        pinegrow.dispatchEvent('on_projectdb_changed', null, col, info || {})
    }

    this.createNode = function(tag, attrs, no_kids) {
        return c(tag, attrs, no_kids);
    }

    var c = function(tag, attrs, no_kids) {
        var p = new pgParserNode();
        p.tagName = p.tagNameOriginal = tag;
        p.selfClosed = no_kids || false;
        p.closed = true;
        p.closingTag = tag;
        p.isElement = p.isElementOrScript = true;
        p.attrListChanged = true;
        p.attrList = [];

        if(attrs) {
            for (const key in attrs) {
                if (attrs.hasOwnProperty(key)) {
                    let value = attrs[key];

                    var quote = '"';

                    if(value !== null) {
                        if(value.indexOf('"') >= 0) {
                            if(value.indexOf("'") >= 0) {
                                value = pgEncodeAttribute(value);
                            } else {
                                quote = "'";
                            }
                        }
                    }
                    p.attrList.push({
                        name: key,
                        value: value,
                        quote: quote,
                        name_lc: key.toLowerCase()
                    })
                }
            }
        }

        if(assign_ids) {
            p.assignId();
            p.nodeCatalogue.add(p);
        }

        return p;
    }

    var onUndoRedo = function(pagenull, info) {
        if(undo_page && info && info.pages) {
            for(var i = 0; i < info.pages.length; i++) {
                if(info.pages[i] === undo_page) {
                    _this.changed(info.pgel_col, {
                        source: 'undo',
                        undo_info: info
                    })
                    break;
                }
            }
        }
    }

    pinegrow.addEventHandler('on_undo_redo', onUndoRedo);

    var onPageSaved = function() {
        var p = pinegrow.getCurrentProject();
        if(p && p.db && p.db.is_changed) {
            p.db.save();
        }
    }

    pinegrow.addEventHandler('on_page_saved', onPageSaved);

    //General ProjectDB api
    this.getDBNode = function(tag, parent, create) {
        parent = parent || project_node;
        var n = parent.findOne('>' + tag);
        if(!n) {
            n = c(tag);
            parent.append(n);
            //trigger change?
        }
        return n;
    }

    //End General ProjectDB api

    this.destroy = function() {
        pinegrow.removeEventHandler('on_undo_redo', onUndoRedo);
        pinegrow.removeEventHandler('on_page_saved', onPageSaved);
        docs_url_map.clear();
        docs_url_map = null;
    }
}
